unit PlayerConvertUnit;

interface

type
  TGTAEdition = (geUndefined, geClassic, geRetail);

function GetEdition(const FileName: string): TGTAEdition;
procedure ConvertTo(const FileName: string; targetEdition: TGTAEdition);

implementation

uses
  SysUtils,
  Classes;

type
  TConvertRule = record
    Position: integer;
    NullBytesAfter: integer;
    constructor Create(APosition: integer; ANullBytesAfter: integer);
  end;

const
  TARGETSIZE_CLASSIC = $414;
  TARGETSIZE_RETAIL  = $3F1;
  DELPHI_STR_BASE = 1;

var
  ConvertArray: array[0..28] of TConvertRule;

function LoadFileToStr(const FileName: TFileName): AnsiString;
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    if FileStream.Size>0 then
    begin
      SetLength(Result, FileStream.Size);
      FileStream.Read(Pointer(Result)^, FileStream.Size);
    end;
  finally
    FileStream.Free;
  end;
end;

procedure SaveStrToFile(const FileName: TFileName; SourceString: AnsiString);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    Stream.WriteBuffer(Pointer(SourceString)^, Length(SourceString));
  finally
    Stream.Free;
  end;
end;

constructor TConvertRule.Create(APosition, ANullBytesAfter: integer);
begin
  Self.Position := APosition;
  Self.NullBytesAfter := ANullBytesAfter;
end;

procedure InitConvertArray;
begin
  ConvertArray[ 0] := TConvertRule.Create(   9, 3); // deathmatch mode (byte->dword)
  ConvertArray[ 1] := TConvertRule.Create(  18, 3); // language (byte->dword)
  ConvertArray[ 2] := TConvertRule.Create(  37, 1); // highscore 1:1 name (15->16 string)
  ConvertArray[ 3] := TConvertRule.Create(  56, 1); // highscore 1:2 name (15->16 string)
  ConvertArray[ 4] := TConvertRule.Create(  75, 1); // highscore 1:3 name (15->16 string)
  ConvertArray[ 5] := TConvertRule.Create(  94, 1); // highscore 2:1 name (15->16 string)
  ConvertArray[ 6] := TConvertRule.Create( 113, 1); // highscore 2:2 name (15->16 string)
  ConvertArray[ 7] := TConvertRule.Create( 132, 1); // highscore 2:3 name (15->16 string)
  ConvertArray[ 8] := TConvertRule.Create( 151, 1); // highscore 3:1 name (15->16 string)
  ConvertArray[ 9] := TConvertRule.Create( 170, 1); // highscore 3:2 name (15->16 string)
  ConvertArray[10] := TConvertRule.Create( 189, 1); // highscore 3:3 name (15->16 string)
  ConvertArray[11] := TConvertRule.Create( 208, 1); // highscore 4:1 name (15->16 string)
  ConvertArray[12] := TConvertRule.Create( 227, 1); // highscore 4:2 name (15->16 string)
  ConvertArray[13] := TConvertRule.Create( 246, 1); // highscore 4:3 name (15->16 string)
  ConvertArray[14] := TConvertRule.Create( 265, 1); // highscore 5:1 name (15->16 string)
  ConvertArray[15] := TConvertRule.Create( 284, 1); // highscore 5:2 name (15->16 string)
  ConvertArray[16] := TConvertRule.Create( 303, 1); // highscore 5:3 name (15->16 string)
  ConvertArray[17] := TConvertRule.Create( 322, 1); // highscore 6:1 name (15->16 string)
  ConvertArray[18] := TConvertRule.Create( 341, 1); // highscore 6:2 name (15->16 string)
  ConvertArray[19] := TConvertRule.Create( 360, 1); // highscore 6:3 name (15->16 string)
  ConvertArray[20] := TConvertRule.Create( 375, 1); // player 1 name (15->16 string)
  ConvertArray[21] := TConvertRule.Create( 454, 1); // player 2 name (15->16 string)
  ConvertArray[22] := TConvertRule.Create( 533, 1); // player 3 name (15->16 string)
  ConvertArray[23] := TConvertRule.Create( 612, 1); // player 4 name (15->16 string)
  ConvertArray[24] := TConvertRule.Create( 691, 1); // player 5 name (15->16 string)
  ConvertArray[25] := TConvertRule.Create( 770, 1); // player 6 name (15->16 string)
  ConvertArray[26] := TConvertRule.Create( 849, 1); // player 7 name (15->16 string)
  ConvertArray[27] := TConvertRule.Create( 928, 1); // player 8 name (15->16 string)
  ConvertArray[28] := TConvertRule.Create(1009, 3); // Multiplayer host title (16->19 string)
end;

function GetEditionFromContent(const Content: AnsiString): TGTAEdition;
begin
       if Length(Content) = TARGETSIZE_RETAIL  then result := geRetail
  else if Length(Content) = TARGETSIZE_CLASSIC then result := geClassic
  else result := geUndefined;
end;

function GetEdition(const FileName: string): TGTAEdition;
begin
  result := GetEditionFromContent(LoadFileToStr(FileName));
end;

procedure ConvertTo(const FileName: string; targetEdition: TGTAEdition);
var
  Target: AnsiString;
  currentEdition: TGTAEdition;
  i: integer;
  Nulls: string;
begin
  Target := LoadFileToStr(FileName);
  currentEdition := GetEditionFromContent(Target);

  if (currentEdition = geRetail) and (targetEdition = geClassic) then
  begin
    // Retail => Classic
    for i := High(ConvertArray) downto Low(ConvertArray) do
    begin
      Nulls := StringOfChar(#0, ConvertArray[i].NullBytesAfter);
      Insert(Nulls, Target, ConvertArray[i].Position+DELPHI_STR_BASE);
    end;
    Assert(Length(Target) = TARGETSIZE_CLASSIC);
    SaveStrToFile(FileName, Target);
  end;

  if (currentEdition = geClassic) and (targetEdition = geRetail) then
  begin
    // Classic => Retail
    for i := Low(ConvertArray) to High(ConvertArray) do
    begin
      Delete(Target, ConvertArray[i].Position+DELPHI_STR_BASE, ConvertArray[i].NullBytesAfter);
    end;
    Assert(Length(Target) = TARGETSIZE_RETAIL);
    SaveStrToFile(FileName, Target);
  end;
end;

begin
  InitConvertArray;
end.
